//========================================================
// Fichier : boltzmannBGK.sci
// Language : Scilab
//
//       Résolution de l'equation Boltzmann-BGK 1D
//========================================================
clear;

//*** Function feq()
//*** Fonction de distribution d'équilibre.
// Liste des arguments :
// Mi : Moments de f.
//      - M0 : Densité.
//      - M1 : Impulsion.
//      - M2 : Energie totale.
// ci : Noeuds des l'espace des vitesses.
// wi : Poids de la quadrature.
function [fi] = feq(M0,M1,M2,cw)
  rho = M0; 
  u = M1./rho;
  theta = (M2 - (u.^2)./rho)./rho;
  
// feq pour N=2
//deff( '[y] = maxwellienne(c)', '[y] = rho.*(1.+ c*u + (c*u).^2/2. + (theta-1)*(c.^2-1)/2. -u.^2/2.)');
  
// feq pour N=3
deff( '[y] = maxwellienne(c)', '[y] = rho.*(1.+ c*u + (c*u).^2/2. +  (theta-1.)*(c^2-1)/2. -u.^2/2. + (c*u).^3/6.- (c*u).*u.^2/2. + (theta-1.).*(c.^2-2).*(c.*u)./2.)');
  
  for i= 1:length(cw(1,:)),
    fi(:,i) = cw(2,i).*maxwellienne(cw(1,i));
  end;
endfunction

//********************************************************//
//                ^^ Initialisation ^^                    //
//********************************************************//

// Définition des paramètres de la simulation 
// Mi0  : Moments initiaux
//      - M00 : Densité.
//      - M10 : Impulsion.
//      - M20 : Energie totale.
// tau : Temps de relaxation vers l'équilibre.
// Nx  : Nombre de noeuds en configurations.
// L   : Longueur de l'espace des configuration.
// x : Variable de configurations (position).
// Nv  : Nombre de noeuds en vitesses.  
// k   : Variable de Fourier.
// k0  : Vecteur d'onde du mode fondamantal.
// dt  : Pas de temps.
// Tf  : Temps discret final.

// Quadrature de G-H.

exec("GaussHermite.sci");
Nv=9; cw = [racines(hermite(Nv))' ; poids(hermite(Nv))'];

// Schemas composites
//cw = [[racines(hermite(Nv)); racines(hermite(Nv+1))]' ; [poids(hermite(Nv))./2;poids(hermite(Nv+1))./2]']; Nv=2*Nv+1;

L  = 20; 
Nx = 5*L;  
x  = L*[1:Nx]'/Nx; 
dx = L/Nx; 
k0 = 2*%pi/L;
k  = k0*[-[0:Nx/2],[Nx/2-1:-1:1]]';
Tf = Nx/2;
dt = (L/Nx)/max(real(cw(1,:))); // Condition CFL (dx/dt=c_{max})
KnMax = (2)*1
Tf = KnMax/dt;
//Tf = 1;

// ** Definition des moments initiaux ** //
//=== Densité initiale (M00 = rho).
rho0  = 1.; // Densité de référence.
// Densité initiale constante :
//M00 = rho0; 

M0 = zeros(Nx,Tf(1));
M1 = zeros(Nx,Tf(1));
M2 = zeros(Nx,Tf(1));

// Perturbation gaussienne d'amplitude alpha de la densité initiale :
alpha = .01; M00 = rho0 * (1. + alpha.*exp(-((x-L/2).^2)/(2))); 

// Perturbation sinusoidale d'amplitude alpha de la densité initiale :
//alpha = .1; M00 = rho0 * (1 + alpha*sin(4*%pi*x/L)); 

//=== Impulsion initiale.
// M10 = rho*u0
u0 = .0; // Vitesse initiale
M10 = M00.*u0;

//=== Energie totale initiale.
theta0 = 1.; // "Température" de référence (imposée à 1 du fait de la quadrature)
// M2 = D*rho*theta + rho*u^2
// Rmq. : theta = k_B * T / m  et D=1 (Dimension de l'espace).
M20 = M00*theta0 + (M10.^2)./M00;

// Condition de Pression constante :
//theta = theta0./M00; M20 = M00.*theta + (M10.^2)./M00;

//****************************************//
//             ^^  Simulation  ^^         //
//****************************************//
set figure_style new;
fig=gcf(); scf(0); clf();

// Initialisation de la fonction de distribution.
f = feq(M00,M10,M20,cw);

// Boucle principale.
for t = 1:Tf,

  // Transport //
  for i=1:Nv, 
    fk(:,i) = fft(f(:,i));
    f(:,i)  = real(ifft(fk(:,i).*exp(%i*k*cw(1,i)*dt)));
  end;  
  
  // Calcul des Moments //
  M00 = sum(f,'c');
  M0(:,t) = M00(:);
  
  for i=1:Nx,
    M10(i) = sum((cw(1,:)).*f(i,:),'c');
  end;

  for i=1:Nx,
    M20(i) = sum(((cw(1,:)).^2).*f(i,:),'c');
  end;

  // Affichage // 
  tr = (t)*dt; 
  drawlater();
  delete(gce());
 //plot(x,1+alpha.*exp(-(x-L/2).^2/(2.*(1.+tr^2)))./sqrt(1.+tr^2),x,1+alpha.*(x-L/2).*tr.*exp(-(x-L/2).^2/(2.*(1.+tr.^2)))./sqrt((1.+tr^2)^3),x,1+alpha.*(1+tr.^2+((x-L/2).*tr).^2).*exp(-(x-L/2).^2/(2.*(1.+tr.^2)))./sqrt((1.+tr.^2).^5));
  
  plot(x,M00,'.',x,1+alpha.*exp(-(x-L/2).^2/(2.*(1.+tr^2)))./sqrt(1.+tr^2),x,M10./M00+1,'+',x,1+alpha.*(x-L/2).*tr.*exp(-(x-L/2).^2/(2.*(1.+tr.^2)))./sqrt((1.+tr^2)^3),x,(M20 - M10.*M10),'.',x,1+alpha.*(1+tr.^2+((x-L/2).*tr).^2).*exp(-(x-L/2).^2/(2.*(1.+tr.^2)))./sqrt((1.+tr.^2).^5));
  drawnow();
  
end; 
