//========================================================
// Fichier : boltzmannBGK.sci
// Language : Scilab
//
// Résolution de l'equation Boltzmann-BGK 1D
//========================================================
clear;

//*** Function feq()
//*** Fonction de distribution d'équilibre.
// Liste des arguments :
// Mi : Moments de f.
//      - M0 : Densité.
//      - M1 : Impulsion.
//      - M2 : Energie totale.
// ci : Noeuds des l'espace des vitesses.
// wi : Poids de la quadrature.
function [fi] = feq(M0,M1,M2,cw)
rho = M0; 
u = M1./rho;
theta = (M2 - (u.^2)./rho)./rho;

// feq pour N=0
//deff( '[y] = maxwellienne(c)', '[y] = rho');

// feq pour N=1
//deff( '[y] = maxwellienne(c)', '[y] = rho.*(1.+ c*u)');

// feq pour N=2
//deff( '[y] = maxwellienne(c)', ...
//'[y] = rho.*(1.+ c*u + (c*u).^2/2. + (theta-1)*(c.^2-1)/2. -u.^2/2.)');

// feq pour N=2 (theta=theta_0)
//deff( '[y] = maxwellienne(c)','[y] = rho.*(1.+ c*u + (c*u).^2/2. -u.^2/2.)');

// feq pour N=3
deff( '[y] = maxwellienne(c)', '[y] = rho.*(1.+ c*u + (c*u).^2/2. +  (theta-1.)*(c.^2-1)/2. -u.^2/2. + (c*u).^3/6.- (c*u).*u.^2/2. + (theta-1.).*(c.^2-2).*(c.*u)./2.)');

for i= 1:length(cw(1,:)),
  fi(:,i) = cw(2,i).*maxwellienne(cw(1,i));
end;
endfunction

//********************************************************
//*** Initialisation
//********************************************************

// Définition des paramètres de la simulation 
// Mi  : Moments initiaux
//      - M0 : Densité.
//      - M1 : Impulsion.
//      - M2 : Energie totale.
// tau : Temps de relaxation vers l'équilibre.
// Nx  : Nombre de noeuds en configurations.
// L   : Longueur de l'espace des configuration.
// x   : Variable de configurations (position).
// dx  : Pas d'espace.
// Nv  : Nombre de noeuds en vitesses.  
// k   : Variable de Fourier.
// k0  : Vecteur d'onde du mode fondamantal.
// dt  : Pas de temps.
// Tf  : Temps discret final.
// Nv  : Nombre de vitesses discretes.
eta = 0.
//*** Quadrature de G-H.
exec("GaussHermite.sci");
Nv = 3; 
//* Schemas 'Pures' :
cw = [racines(hermite(Nv))' ; poids(hermite(Nv))'];

//* Schemas composites :
eta = 0.5;cw = [[racines(hermite(Nv)); racines(hermite(Nv+1))]' ; [eta*poids(hermite(Nv));(1-eta).*poids(hermite(Nv+1))]']; Nv=2*Nv+1; // Nombre de vitesses discretes du schema composites.


L  = 30; 
Nx = 2*L;
x  = L*[1:Nx]'/Nx; 
dx = L/Nx;
k0 = 2*%pi/L;
k  = k0*[-[0:Nx/2],[Nx/2-1:-1:1]]';
dt = (L/Nx)/max(real(cw(1,:))); // Condition CFL : dx/dt <= c_{max} (= ici).
tau = 1*dt*ones(Nx,Nv); // tau >= .5*dt
Tf = L./dt;

// ** Definition des moments initiaux ** //

//=== Densité initiale (M00 = rho).
rho0  = 1.; // Densité de référence.
// Densité initiale constante :
//M0 = rho0.*ones(Nx,1); 

// Perturbation gaussienne d'amplitude alpha de la densité initiale :
alpha = .01; M0 = rho0 * (1 - alpha.*exp(-(((x)-L/2).^2)/(4))); 

// Perturbation sinusoidale d'amplitude alpha de la densité initiale :
//alpha = .1; M0 = rho0 * (1 + alpha*sin(2*%pi*(x-1)/(L))); 

//=== Impulsion initiale.
// M1 = rho*u0
u0 = 0;//.05*dt./dt; // Vitesse initiale
M1 = M0.*u0;

//=== Energie totale initiale.
theta0 = 1.; // 'Température' de référence (imposée à 1 par la quadrature).
// M2 = D*rho*theta + rho*u^2
// Rmq. : theta = k_B * T / m  et D = 1 (Dimension de l'espace).

// Condition d'équilibre themodynamique :
//M2 = M0*theta0 + (M1.^2)./M0; theta = (theta0./M0).*M0;

// Condition de Pression constante :
theta = theta0./M0; M2 = M0.*theta + (M1.^2)./M0;

// Diffusivité thermique.
D = zeros(Nx,1);

// Vairiables pour la résolution de l'équation de la chaleur :
temp = theta; tempTmp = temp;

//********************************************************
//*** Simulation
//********************************************************
set figure_style new;
fig=gcf();
clf();

// Initialisation de la fonction de distribution.
f = feq(M0,M1,M2,cw);

unix_g('rm therm.res')
fd = mopen("./therm.res",'w'); 
mfprintf(fd,'#Nv=%d, Tau=%f, dt=%f\n',Nv,tau,dt);   

// Boucle principale
for t = 1:Tf,

  //** Résolution de l'équation de Boltzmann-BGK
  // Transport
  for i=1:Nv, 
    fk(:,i) = fft(f(:,i));
    f(:,i)  = real(ifft(fk(:,i).*exp(%i*k*cw(1,i)*dt)));
  end;  
  
  // Calcul des Moments
  M0 = sum(f,'c');
  
  for i=1:Nx,
    M1(i) = sum((cw(1,:)).*f(i,:),'c');
  end;

  for i=1:Nx,
    M2(i) = sum(((cw(1,:)).^2).*f(i,:),'c');
  end;
  theta = (M2 - M1.*M1)./M0;
  
  // Calcul du temps de relaxation local
  kapa=1/2; // Exposant thermique du gaz (1/2 : spheres dures) 
  for i=1:Nv,
    tau2(:,i) = (rho0./M0).*((theta0./theta).^(1-kapa)).*tau(i);
  end;
  
  // Collisions 
  f = f - (dt./(tau2)).*(f - feq(M0,M1,M2,cw)); 
  
  //** Résolution de l'équation de la chaleur 
  // Calcul de la diffusivite thermique locale 
  D = (1-eta)*theta.*(tau2(:,1)-dt./2);
  
  tempTmp(1) = temp(1) + temp(1)*D(1)*(dt/dx^2)*(temp(2) + temp(Nx) -2*temp(1));  
  tempTmp(Nx) = temp(Nx) + temp(Nx)*D(Nx)*(dt/dx^2)*(temp(1) + temp(Nx-1) -2*temp(Nx));
  for i=2:(Nx-1),
    tempTmp(i)  = temp(i) + temp(i)*D(i)*(dt/dx^2)*(temp(i+1) + temp(i-1) -2*temp(i));
  end
  temp = tempTmp; 
  
  
  //** Ecriture du résultat et affichage 
  mfprintf(fd,'#Index %d, t = %f\n',t,(t-1)*dt);    
  for i=1:Nx,
    //mfprintf(fd,'%f %f %f\n',i*dx,(M2(i) - M1(i)*M1(i))./M0(i),theta(i));
    mfprintf(fd,'%f %f %f %f %f\n',i*dx,theta(i),temp(i),M0(i),M0(i).*theta(i));
  end
  mfprintf(fd,'\n\n');
  
  drawlater();
  delete(gce());
  plot(x,theta,'.',x,temp,x,M0,'+',x,M0.*theta);  
  drawnow();
  
end; // Fin de la boucle principale.

mclose (fd);
