//========================================================
// Fichier : boltzmannBGK.sci
// Language : Scilab
//
// Résolution de l'equation Boltzmann-BGK 1D
//========================================================
clear;

//*** Function feq()
//*** Fonction de distribution d'équilibre.
// Liste des arguments :
// Mi : Moments de f.
//      - M0 : Densité.
//      - M1 : Impulsion.
//      - M2 : Energie totale.
// ci : Noeuds des l'espace des vitesses.
// wi : Poids de la quadrature.
function [fi] = feq(M0,M1,M2,cw)
rho = M0; 
u = M1./rho;
theta = (M2 - (u.^2)./rho)./rho;

// feq pour N=0
//deff( '[y] = maxwellienne(c)', '[y] = rho');

// feq pour N=1
//deff( '[y] = maxwellienne(c)', '[y] = rho.*(1.+ c*u)');

// feq pour N=2
//deff( '[y] = maxwellienne(c)','[y] = rho.*(1.+ c*u + (c*u).^2/2. + (theta-1)*(c.^2-1)/2. -u.^2/2.)');

// feq pour N=2 (theta=theta_0)
//deff( '[y] = maxwellienne(c)','[y] = rho.*(1.+ c*u + (c*u).^2/2. -u.^2/2.)');

// feq pour N=3
deff( '[y] = maxwellienne(c)', '[y] = rho.*(1.+ c*u + (c*u).^2/2. +  (theta-1.)*(c.^2-1)/2 -u.^2/2. + (c*u).^3/6.- (c*u).*u.^2/2. + (theta-1.).*(c.^2-2).*(c.*u)/2)');


for i= 1:length(cw(1,:)),
  fi(:,i) = cw(2,i).*maxwellienne(cw(1,i));
end;
endfunction

//********************************************************
//*** Initialisation
//********************************************************

// Définition des paramètres de la simulation 
// Mi  : Moments initiaux
//      - M0 : Densité.
//      - M1 : Impulsion.
//      - M2 : Energie totale.
// tau : Temps de relaxation vers l'équilibre.
// Nx  : Nombre de noeuds en configurations.
// L   : Longueur de l'espace des configuration.
// x   : Variable de configurations (position).
// dx  : Pas d'espace.
// Nv  : Nombre de noeuds en vitesses.  
// k   : Variable de Fourier.
// k0  : Vecteur d'onde du mode fondamantal.
// dt  : Pas de temps.
// Tf  : Temps discret final.
// Nv  : Nombre de vitesses discretes.

//*** Quadrature de G-H.
exec("GaussHermite.sci");
Nv = 3; 
//* Schemas 'Pures' :
cw = [racines(hermite(Nv))' ; poids(hermite(Nv))'];

//* Schemas composites :
//eta = 0.5; cw = [[racines(hermite(Nv)); racines(hermite(Nv+1))]' ; [eta*poids(hermite(Nv));(1-eta).*poids(hermite(Nv+1))]']; Nv=2*Nv+1; // Nombre de vitesses discretes du schema composites.

L  = 100; 
Nx = L;
x  = L*[1:Nx]'/Nx; 
dx = L/Nx;
k0 = 2*%pi/L;
k  = k0*[-[0:Nx/2],[Nx/2-1:-1:1]]';
dt = (L/Nx)/max(real(cw(1,:))); // Condition CFL : dx/dt <= c_{max} (= ici).
tau = 1.*dt*ones(Nx,Nv); // tau >= .5*dt
Tf = L/dt;
alpha = .001*dx/dt;

// ** Definition des moments initiaux ** //

//=== Densité initiale (M00 = rho).
rho0  = 1.; // Densité de référence.
// Densité initiale constante :
M0 = rho0.*ones(Nx,1); 

// Perturbation gaussienne d'amplitude alpha de la densité initiale :
//alpha = .01; M0 = rho0 * (1. - alpha.*exp(-(((x)-L/2).^2)/(4))); 

// Perturbation sinusoidale d'amplitude alpha de la densité initiale :
//alpha = .1; M0 = rho0 * (1 + alpha*sin(2*%pi*(x-1)/(L))); 

//=== Impulsion initiale.
// M1 = rho*u0
//u0 = 0;//.05*dt./dt; // Vitesse initiale
//M1 = M0.*u0;
Nk=5;
M1 = alpha*sin(Nk*k0*(x-1)); 

//=== Energie totale initiale.
theta0 = 1.; // 'Température' de référence (imposée à 1 du fait de la quadrature).
// M2 = D*rho*theta + rho*u^2
// Rmq. : theta = k_B * T / m  et D = 1 (Dimension de l'espace).

// Condition d'équilibre themodynamique :
M2 = M0*theta0 + (M1.^2)./M0; theta = (theta0./M0).*M0;

// Condition de Pression constante :
//theta = theta0./M0; M2 = M0.*theta + (M1.^2)./M0;
M3 = 0.*ones(Nx,1); 

ETh = zeros(Tf,1);
U2 = zeros(Tf,1);
//********************************************************
//*** Simulation
//********************************************************
set figure_style new;
fig=gcf();
//fig.pixmap='on';
clf();

// Initialisation de la fonction de distribution.
f = feq(M0,M1,M2,cw);
  

// Boucle principale
for t = 1:Tf,
 U2(t) =  sum(real(M1.*M1./M0));
 theta = M2 - M1.*M1./M0;
 ETh(t)= sum(sqrt(theta.*theta)) - Nx;
  
  //** Résolution de l'équation de Boltzmann-BGK
  // Transport
  for i=1:Nv, 
    fk(:,i) = fft(f(:,i));
    f(:,i)  = real(ifft(fk(:,i).*exp(%i*k*cw(1,i)*dt)));
  end;  
  
  // Calcul des Moments
  M0 = sum(f,'c');
  
  for i=1:Nx,
    M1(i) = sum((cw(1,:)).*f(i,:),'c');
  end;
 
  for i=1:Nx,
    M2(i) = sum(((cw(1,:)).^2).*f(i,:),'c');
  end;
  //theta = (M2 - (u.^2)./rho)./rho;
 
  for i=1:Nx,
    M3(i) = sum(((cw(1,:)).^3).*f(i,:),'c');
  end;
  q = M3 - 3*(M1).*(M2);
  
  // Calcul du temps de relaxation local
  kapa=1/2; // Exposant thermique du gaz [1/2;1] (1/2 : spheres dures) 
  for i=1:Nv,
    tau2(:,i) = (rho0./M0).*((theta0./theta).^(1-kapa)).*tau(i);
  end;
  
  // Collisions 
  f = f - (dt./(tau2)).*(f - feq(M0,M1,M2,cw)); 
  

  drawlater();
  delete(gce());
  //sleep(100)
  //plot(x,theta,'.',x,M0,'+',x,M1./M0+1);  
  plot(dt.*(0:Tf-1)',U2+ETh,dt.*(0:Tf-1)',U2,dt.*(0:Tf-1)',ETh,dt.*(0:Tf-1)',U2(1).*exp(-dt.*(0:Tf-1)'.*(2*(Nk*k0).^2.*(tau(1,1)-dt/2))));
  drawnow();
  
end; // Fin de la boucle principale.

//clf();
